# -*- coding: utf-8 -*-
#!/usr/bin/env python
import logging

import os
import codecs
import vobject
import StringIO

import qrcode

from docutils.core import publish_parts
from flask import Flask
from flask import render_template
from flask import request
from flask_babel import gettext as _
from flask_babel import Babel
from config import LANGUAGES

import base64

LANGUAGE_SELECTED = "de"
# ToDo after engelish is implemented set LANGUAGE_SELECTED = None

# We need the path of this file to find templates to translate
ESKP_PATH = os.path.dirname(os.path.abspath(__file__))
logging.basicConfig(filename=os.path.join(ESKP_PATH, 'eskp-app.log'), level=logging.DEBUG)
FILES= os.listdir(os.path.join(ESKP_PATH, 'static/images/uvmap'))

app = Flask(__name__)
babel = Babel(app)

app.config['BABEL_DEFAULT_LOCALE'] = 'de'


def get_vcard(filename):
    filename = os.path.join(ESKP_PATH, filename)
    with codecs.open(filename, 'r', 'utf-8') as f:
        vcard = f.read()
    return vobject.readOne(vcard)


def get_content(filename, overrides=None):
    content = u""
    filename = os.path.join(ESKP_PATH, filename)
    if os.path.isfile(filename):
        with codecs.open(filename, 'r', 'utf-8') as f:
            rst_data = f.read()
        content = publish_parts(rst_data, writer_name='html', settings_overrides=overrides)['html_body']
    return content

def get_html_content(filename, overrides=None):
    html_content = u""
    filename = os.path.join(ESKP_PATH, filename)
    if os.path.isfile(filename):
        with codecs.open(filename, 'r', 'utf-8') as f:
            html_content = f.read()

    return html_content

def get_newest_date():
    getdates = get_valid_dates(FILES)
    newest_date = getdates[-1]

    return newest_date


def get_topmenue():
    newest_date = get_newest_date()
    menue = [
        ('/ozoneloss', _(u'Ozoneloss'),
         (('/ozoneloss', _(u'overview')),
          ('/ozoneloss/clams/2022', _(u'calculations')),
          ('/ozoneloss/vpsc/2022', _(u'estimations')),
          ('/ozoneloss/uvmap/' + newest_date, _(u'uv index map')),
          ('/ozoneloss/uvi', _(u'uv increase')),
          ('/ozoneloss/decadal', _(u'decadal')))),
        ('/eskp', _(u'ESKP'), (None, None)),
        ('/iek-7', _(u'IEK-7'), (None, None))
    ]
    return menue


app.jinja_env.globals.update(get_topmenue=get_topmenue)


def get_o3lossclams_dates():
    menue = [
        ('/ozoneloss/clams/2022', _(u'2022')),
        ('/ozoneloss/clams/2020', _(u'2020')),
        ('/ozoneloss/clams/2019', _(u'2019')),
        ('/ozoneloss/clams/2018', _(u'2018')),
        ('/ozoneloss/clams/2017', _(u'2017')),
        ('/ozoneloss/clams/2016', _(u'2016')),
        ('/ozoneloss/clams/2015', _(u'2015')),
        ('/ozoneloss/clams/2012', _(u'2012')),
        ('/ozoneloss/clams/2011', _(u'2011')),
        ('/ozoneloss/clams/2010', _(u'2010')),
    ]
    return menue


def get_valid_dates(files):
    dates = []

    for file in files:
        if file.endswith('.png') and file.find('uvi') >= 0:
            date = file[-12:-6]
            dates.append(date)
    dates.sort()
    for date in dates:
        i = 0
        for param in ['uvi', 'o3col', 'do3col']:
            testfile = 'clams_' + param + '_' + date + '12.png'
            if files.count(testfile) > 0:
                i = i + 1
        if i <> 3:
            dates.remove(date)
    return dates


def get_o3lossuvmap_dates(date_show):
    dates = get_valid_dates(FILES)
    ndates = len(dates)
    ind = dates.index(date_show)
    navitexts = []
    if ind ==0 :
        chosendates = [dates[ind+1]]
        navitexts.append('next ->')

    elif ind >= ndates - 1:
        chosendates= [dates[ind-1]]
        navitexts.append('<- prev')
    else:
        chosendates = [dates[ind-1], dates[ind+1]]
        navitexts.append('<- prev')
        navitexts.append('next ->')
    menue = []

    for i in range(len(chosendates)):
        date = chosendates[i]
        navitext = navitexts[i]
        menue.append(('/ozoneloss/uvmap/' + date, _(navitext)))
    return menue


def get_vpsc_dates():
    menue = [
        ('/ozoneloss/vpsc/2022', _(u'2022')),
        ('/ozoneloss/vpsc/2020', _(u'2020')),
        ('/ozoneloss/vpsc/2019', _(u'2019')),
        ('/ozoneloss/vpsc/2018', _(u'2018')),
        ('/ozoneloss/vpsc/2017', _(u'2017')),
        ('/ozoneloss/vpsc/2016', _(u'2016')),
        ('/ozoneloss/vpsc/2015', _(u'2015')),
        ('/ozoneloss/vpsc/2014', _(u'2014')),
        ('/ozoneloss/vpsc/2013', _(u'2013')),
        ('/ozoneloss/vpsc/2012', _(u'2012')),
        ('/ozoneloss/vpsc/2011', _(u'2011')),
        ('/ozoneloss/vpsc/2010', _(u'2010')),
    ]
    return menue


app.jinja_env.globals.update(get_o3lossclams_dates=get_o3lossclams_dates)
app.jinja_env.globals.update(get_vpsc_dates=get_vpsc_dates)
app.jinja_env.globals.update(get_o3lossuvmap_dates=get_o3lossuvmap_dates)


def modal_info(template, act, title, filename):
    content = get_content(filename)
    html = render_template(template, act=act, title=title, content=content, exit=_(u"Close"))
    return html


@babel.localeselector
def get_locale():
    requested_language = request.accept_languages.best_match(LANGUAGES.keys())
    if requested_language in ("de", "en"):
        return requested_language or "de"

@app.route("/")
@app.route("/index")
def index():
    return render_template("/index.html",
                           eskp_info=_(u'About ESKP'),
                           )


@app.route('/ozoneloss/clams/<year>')
def ozoneloss_clams_year(year):
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_clams.rst")
    content = get_content(filename)
    return render_template("/ozoneloss_clams.html", act="ozoneloss/clams/%s" % year, content=content, year=year)


@app.route('/ozoneloss/uvmap/<date>')
def ozoneloss_uvmap_date(date):
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_uvmap.rst")
    content = get_content(filename)
    htmlfile = os.path.join("templates", get_locale(), "html", "uvi_table.html")
    html_table = get_html_content(htmlfile)

    return render_template("/ozoneloss_uvmap.html", act="ozoneloss/uvmap/%s" % date, content=content,
                           html_table=html_table, date=date)


@app.route('/ozoneloss/vpsc/<year>')
def ozoneloss_vpsc_year(year):
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_vpsc.rst")
    content = get_content(filename)
    filename = os.path.join("templates", get_locale(), "rst", "explanation_vpsc.rst")
    explanation = get_content(filename)

    return render_template("/ozoneloss_vpsc.html", act="ozoneloss/vpsc/%s" % year, content=content,
                           content_explanation=explanation, year=year)


@app.route('/de')
def de():
    global LANGUAGE_SELECTED
    LANGUAGE_SELECTED = "de"
    return render_template("/index.html",
                           eskp_info=_(u'About ESKP'),
                           )


@app.route('/en')
def en():
    global LANGUAGE_SELECTED
    LANGUAGE_SELECTED = "en"
    return render_template("/index.html",
                           eskp_info=_(u'About ESKP'),
                           )


@app.route("/eskp")
def eskp():
    filename = os.path.join("templates", get_locale(), "rst", "eskp.rst")
    content = get_content(filename)
    filename = os.path.join("templates", get_locale(), "rst", "eskp_title.rst")
    headline = get_content(filename)
    return render_template("/eskp.html", act="eskp", content=content, headline=headline)


def qr_image_data(card):
    buf = StringIO.StringIO()
    qr = qrcode.QRCode(
        version=1,
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=2,
        border=2,
    )
    qr.add_data(card.serialize())
    qr.make(fit=True)
    img = qr.make_image()
    img.save(buf)
    image = buf.getvalue()
    return base64.b64encode(image)


@app.route("/ozoneloss")
def ozoneloss():
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss.rst")
    content = get_content(filename)
    ref_date = "200119"
    newest_date = get_newest_date()
    content = content.replace(ref_date, newest_date, 1)

    vcard_file = os.path.join("vcards", "jug.vcf")
    author = u""

    try:
        card = get_vcard(vcard_file)
    except IOError:
        card = None
    if card is not None:
        qr_image = qr_image_data(card)
        author = render_template("/author_info.html", act="author", title=_(u"Ozoneloss"),
                                 card=card, image=qr_image, contact=_(u"Contact"), exit=_(u"Close"))

    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_publications.rst")
    publications = modal_info("/publications_info.html", "publications", _(u"Ozoneloss"), filename)
    return render_template("/ozoneloss.html", act="ozoneloss", content=content,
                           author=author, card=card, publications=publications)


@app.route("/ozoneloss/clams")
def ozoneloss_clams():
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_clams.rst")
    content = get_content(filename)
    return render_template("/ozoneloss_clams.html", act="ozoneloss/clams", content=content)


@app.route("/ozoneloss/uvmap")
def ozoneloss_uvmap():
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_uvmap.rst")
    content = get_content(filename)

    htmlfile = os.path.join("templates", get_locale(), "html", "uvi_table.html")
    html_content = get_html_content(htmlfile)
    return render_template("/ozoneloss_uvmap.html", act="ozoneloss/uvmap", content=content,
                           html_content=html_content)


@app.route("/ozoneloss/vpsc")
def ozoneloss_vpsc():
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_vpsc.rst")
    content = get_content(filename)
    return render_template("/ozoneloss_vpsc.html", act="ozoneloss/vpsc", content=content)


@app.route("/ozoneloss/uvi", methods=['GET'])
def ozoneloss_uvi():
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_uvi.rst")
    content = get_content(filename)
    htmlfile = os.path.join("templates", get_locale(), "html", "ozoneloss_uvi_form_start.html")
    html_content = get_html_content(htmlfile)
    htmlfile = os.path.join("templates", get_locale(), "html", "uvi_table.html")
    html_table = get_html_content(htmlfile)

    lat = 50.
    o3offset = 50.
    figname = "uvincr_lat%0.3i_do3%0.3i.svg" % (lat, o3offset)

    return render_template('ozoneloss_uvi.html', act="ozoneloss/uvi", content=content, figname=figname,
                           alt=_(u"UV increase at {{lat}} degrees N for {{o3offset}} DU ozone depletion"),
                                 html_content=html_content, html_table=html_table)


@app.route("/ozoneloss/uvi_graph", methods=['POST'])
def ozoneloss_uvi_graph():
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_uvi.rst")
    content = get_content(filename)
    htmlfile = os.path.join("templates", get_locale(), "html", "ozoneloss_uvi_form.html")
    html_content = get_html_content(htmlfile)
    htmlfile = os.path.join("templates", get_locale(), "html", "uvi_table.html")
    html_table = get_html_content(htmlfile)

    latstr = request.form['Gradzahl']
    o3offsetstr = request.form['Dobson-Unit']

    latstr2 = latstr.replace(u'\xb0', '')
    if latstr2.endswith(u'N'):
        latstr2 = latstr2.replace(u'N', '')
        lat = float(latstr2)
    if latstr2.endswith(u'S'):
        latstr2 = latstr2.replace(u'S', '')
        lat = -float(latstr2)
        
    html_content = html_content.replace('{{latstr}}', latstr)
    html_content = html_content.replace('{{o3offsetstr}}', o3offsetstr)
    figname = "uvincr_lat%0.3i_do3%0.3i.svg" % (lat, float(o3offsetstr))

    return render_template('ozoneloss_uvi.html', act="ozoneloss_uvi_graph", content=content, figname=figname,
                           alt=_(u"UV increase at {{lat}} degrees N for {{o3offset}} DU ozone depletion"),
                           html_content=html_content, html_table=html_table)

@app.route("/ozoneloss/decadal")
def ozoneloss_decadal():
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_decadal.rst")
    content = get_content(filename)
    filename = os.path.join("templates", get_locale(), "rst", "ozoneloss_decadal_bottom.rst")
    explanation = get_content(filename)
    
    return render_template("/ozoneloss_decadal.html", act="ozoneloss_decadal", content=content,
                           content_explanation=explanation)



@app.route("/iek-7")
def institute():
    filename = os.path.join("templates", get_locale(), "rst", "iek-7.rst")
    content = get_content(filename)
    vcard_file = os.path.join("vcards", "sas.vcf")
    author = u""
    try:
        card = get_vcard(vcard_file)
    except IOError:
        card = None
    if card is not None:
        qr_image = qr_image_data(card)
        author = render_template("/author_info.html", act="author", title=_(u"IEK-7"),
                                 card=card, image=qr_image, contact=_(u"Contact"), exit=_(u"Close"))

    return render_template("/iek-7.html", act="iek-7", content=content,
                           author=author, card=card, contact=u"IEK-7")


@app.route("/imprint")
def imprint():
    filename = os.path.join("templates", get_locale(), "rst", "imprint.rst")
    content = get_content(filename)
    return render_template("/content.html", act="imprint", content=content)


@app.route("/dataprotection")
def dataprotection():
    filename = os.path.join("templates", get_locale(), "rst", "datenschutz.rst")
    content = get_content(filename)
    return render_template("/content.html", act="datenschutz", content=content)


@app.errorhandler(404)
def page_not_found(e):
    msg = _(u"Url: %(url)s not found", url=request.url)
    info = _(u"This information is not available!")
    return render_template("404.html", msg=msg, info=info)


if __name__ == "__main__":
    app.run(host='localhost', port=5014)
